<?php

namespace Plugins\embed;

use \Typemill\Plugin;

class embed extends Plugin
{

	protected $pluginData;

	protected $services = [
		"youtube" 		=> [
			'url'  			=> "https://www.youtube.com/watch",
			'privacy' 		=> "https://policies.google.com/privacy",
			'csp'			=> ["https://www.youtube-nocookie.com/", "https://www.youtube.com"]
		],
		"gmaps" 		=> [
			'url'  			=> "https://www.google.com/maps/place",
			'privacy' 		=> "https://policies.google.com/privacy",
			'csp'			=> ["https://maps.google.com/", "https://www.google.com/maps/"]
		],
		"vimeo" 		=> [
			'url'  			=> "https://vimeo.com/",
			'privacy' 		=> "https://vimeo.com/privacy",
			'csp'			=> ["https://player.vimeo.com/"]
		],
		"soundcloud" 	=> [
			'url'  			=> "https://soundcloud.com/",
			'privacy' 		=> "https://soundcloud.com/pages/privacy",
			'csp'			=> ["https://w.soundcloud.com/"]
		],
	];

	protected $error;

	protected $embedFound = false;

	# you can add a licence check here
	public static function setPremiumLicence()
	{
		return 'MAKER';
	}	

	public static function getSubscribedEvents()
	{
		return [
			'onCspLoaded'		=> 'onCspLoaded',
			'onShortcodeFound' 	=> 'onShortcodeFound',
			'onHtmlLoaded' 		=> 'onHtmlLoaded',
		];
	}

	public function onCspLoaded($csp)
	{
		$data = $csp->getData();

		$pluginSettings = $this->getPluginSettings();

		$activeServices = $pluginSettings['services'] ?? false;

		if($activeServices)
		{
			foreach($this->services as $serviceName => $serviceData)
			{
				if(in_array($serviceName, $activeServices))
				{
					foreach($serviceData['csp'] as $domain)
					{
						$data[] = $domain;
					}
				}
			}
		}

		$csp->setData($data);
	}

	public function onShortcodeFound($shortcode)
	{
		$shortcodeArray = $shortcode->getData();

		# register the shortcode
		if(is_array($shortcodeArray) && $shortcodeArray['name'] == 'registershortcode')
		{
			$shortcodeArray['data']['embed'] = [ 'url' => '', 'parameter' => 'example=600;useforheight=350'];

			$shortcode->setData($shortcodeArray);
		}

		if (is_array($shortcodeArray) && $shortcodeArray['name'] == 'embed')
		{
			$shortcode->stopPropagation();

			$url = $shortcodeArray['params']['url'] ?? false;
			if(!$url OR $url == '')
			{
				$html = "Please add a valid embed url.";
			}
			else
			{
				$pluginSettings = $this->getPluginSettings();
				$serviceName = $this->getServiceFromUrl($url);
				if(!$serviceName)
				{
					$html = "The url seems to be wrong and we did not find a way to embed. Please always use the url from the browser bar if you want to embed something. See supported services and example urls here:";
				}
				elseif(!isset($pluginSettings['services']))
				{
					$html = "No services are activated in the plugin settings.";
				}
				elseif(!in_array($serviceName, $pluginSettings['services']))
				{
					$html = "Please check if the service " . $serviceName . " is activated in the plugin settings.";
				}
				else
				{
					$methodName = $serviceName . "Iframe";
					if(method_exists($this, $methodName))
					{
						$parameter 	= $shortcodeArray['params']['parameter'] ?? false;
						$parameter 	= $this->sanitizeParameter($parameter);
						$iframe 	= $this->$methodName($url, $parameter);

						if($iframe)
						{
							$html = '<div 
										class="embedWrapper" 
										data-service="' . $serviceName . '" 
										data-privacylink="' . $this->services[$serviceName]['privacy'] . '" 
										style="margin:auto">' . $iframe . 
									'</div>';
						
							$this->embedFound = true;
						}
						else
						{
							$html = $this->error;
						}
					}
					else
					{
						$html = "Method $methodName does not exist.";
					}
				}
			}
			
			$shortcode->setData($html);
		}
	}

	public function onHtmlLoaded($html)
	{
		$htmlString = $html->getData();

		$pluginSettings = $this->getPluginSettings();

		$privacytext 	= $pluginSettings['privacytext'] ?? "If you want to load content from this service, use the button below and read the ";
		$privacylabel 	= $pluginSettings['privacylabel'] ?? "privacy policy";
		$buttonlabel	= $pluginSettings['buttonlabel'] ?? "load content from this service";

		if($this->embedFound && isset($pluginSettings['privacybox']) && $pluginSettings['privacybox'])
		{
			$this->addJS('/embed/js/script.js');
			$this->addCSS('/embed/css/style.css');

			$htmlString .= '<div 
								id 					= "embedPrivacyConsent" 
								style 				= "display:none"
								data-privacytext	= "' . $privacytext . '"
								data-privacylabel 	= "' . $privacylabel . '"
								data-buttonlabel 	= "' . $buttonlabel . '"
							>';
		}

		$html->setData($htmlString);
	}

	protected function getServiceFromUrl($url)
	{
	    # Cleanup google international
	    if (strpos($url, 'google') !== false)
	    {
	        $url = preg_replace('/\.google\.\w{2,3}(\/|$)/', '.google.com$1', $url);
	    }

		foreach($this->services as $serviceName => $serviceData)
		{
			if(strpos($url, $serviceData['url']) === 0)
			{
				return $serviceName;
			}
		}

		return false;
	}

	protected function youtubeIframe(string $url, array $params)
	{
		preg_match('/(?:\?|&)v=([a-zA-Z0-9_-]+)/', $url, $matches);

		if (isset($matches[1]))
		{
			$id = $matches[1];
		}
		else
		{
			$this->error = 'We could not find a valid id in the url';

			return false;
		}

		$src 		= 'https://www.youtube-nocookie.com/embed/' . $id;
		$width 		= $params['width'] ?? '560';
		$height  	= $params['height'] ?? '315';

		$html = '<iframe 
					width			= "' . $width . '" 
					height 			= "' . $height . '" 
					src 			= "' . $src . '"
					title 			= "YouTube video player" 
					frameborder 	= "0" 
					allow 			= "accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; web-share" 
					referrerpolicy 	= "strict-origin-when-cross-origin" 
					allowfullscreen>
				</iframe>';

		return $html;
	}

	protected function gmapsIframe(string $url, array $params)
	{
		$urlParts = parse_url($url);
		if (!isset($urlParts['path']))
		{
			$this->error = 'We could not find a valid path in the url';

			return false;
		}

		preg_match('/@([-0-9.]+),([-0-9.]+),/', $urlParts['path'], $matches);
		if (!isset($matches[1]) OR !isset($matches[1]) )
		{
			$this->error = 'We could not find valid coordinates (latitude or longitude) in the url';
			
			return false;
		}

		$latitude = $matches[1];
		$longitude = $matches[2];

		preg_match('/\/place\/([^\/?]+)/', $urlParts['path'], $addressMatches);
		$address = urldecode($addressMatches[1]);
		if (!isset($addressMatches[1]))
		{
			$this->error = 'We could not find a valid address in the url, this plugin only supports map-urls for places with a valid address';
		}

		$width 		= $params['width'] ?? '100%';
		$height  	= $params['height'] ?? '450';
		$zoom  		= $params['zoom'] ?? '17';

		$src = 	'https://maps.google.com/maps?'
				. 'width='		. $width
				. '&height=' 	. $height 
				. '&hl=en' 
				. '&coord=' 	. $latitude . ',' . $longitude
				. '&q=' 		. urlencode($address) 
				. '&z=' 		. $zoom 
				. '&iwloc=B&output=embed';

		$html = '<iframe 
					width 			= "' . $width . '" 
					height 			= "' . $height . '" 
					src 			= "' . $src . '" 
					frameborder 	= "0" 
					scrolling 		= "no" 
					marginheight 	= "0" 
					marginwidth 	= "0">
				</iframe>';

		return $html;
	}

	protected function vimeoIframe(string $url, array $params)
	{
		preg_match('/vimeo\.com\/([0-9]+)/', $url, $matches);

		if(isset($matches[1]))
		{
			$id = $matches[1];
		}
		else
		{
			$this->error = 'We could not find a valid id in the vimeo url';

			return false;
		}

		$src 		= 'https://player.vimeo.com/video/' . $id;
		$width 		= $params['width'] ?? '640';
		$height  	= $params['height'] ?? '360';

		$html = '<iframe 
					src 			= "' . $src . '" 
					width 			= "' . $width . '" 
					height 			= "' . $height . '" 
					frameborder 	= "0" 
					allow 			= "autoplay; fullscreen; picture-in-picture" 
					allowfullscreen>
				</iframe>';

		return $html;
	}

	protected function soundcloudIframe(string $url, array $params)
	{
		$oembedUrl = 'https://soundcloud.com/oembed?url=' . urlencode($url) . '&format=json';
		$response = file_get_contents($oembedUrl);

		if($response === false)
		{
		    if (!empty($http_response_header))
		    {
		        list($version, $status_code, $msg) = explode(' ', $http_response_header[0], 3);
		        
				$this->error 	= 'We got an error from soundcloud: ' . $status_code . ' ' . $msg;
		    }
		    else
		    {
				$this->error = 'We did not get any response from soundcloud, the request might be blocked by your server.';
		    }

			return false;
		}

		$oembedData = json_decode($response, true);

		if(isset($oembedData['html']))
		{
			return $oembedData['html'];
		}

		$this->error = 'We could not find anything for the soundcloud url.';

		return false;
	}

	private function sanitizeParameter(string $parameter)
	{
		if($parameter)
		{
			$sanitizedString = preg_replace('/[^a-zA-Z0-9=;]/', '', $parameter);

			$paramsArray = explode(';', $sanitizedString);

			$params = [];
			foreach ($paramsArray as $param)
			{
				if (strpos($param, '=') !== false)
				{
					list($key, $value) = explode('=', $param, 2);
					$params[$key] = $value;
				}
			}

			return $params;
		}

		return [];
	}
}